//////////////////////////////////////////////////////////////////////////
// This program codes an example of using the romberg
// method to estimate the integral 
//
//          I = int[f(x),x=a..b],
//
// where f(x) is a user-defined function.  For this example
// we will assume f(x) is e^x and a=0 b=1.
//                        thomas l. gibson  April 25, 2002
//
//             updated by thomas l. gibson  April 20, 2004
//////////////////////////////////////////////////////////////////////////
#include <cstdlib>               // Needed for system calls
#include <iostream>              // Needed for cout and cin
#include <fstream>               // Needed for file output
#include <iomanip>               // Needed for setw function in file out
#include <cmath>                 // Needed for math functions
#include <cassert>               // Needed for assert
using namespace std;

double f(double t);              // Function prototype
int main()
{
  double a,b,h,error,relerr,sum,x,exact,dnr,endpts;
  double r[20][20];              // Array to hold Richardson extrapolates
  int nr,np,i,j,k,m,doit;
  char ask[2];
  exact=exp(1.0) - 1.0;          // Exact value of integral
  doit=1;
  a=0.0;                         // Lower limit of integration
  b=1.0;                         // Upper limit of integration
  ofstream prt_1("romb_results.dat");
  ofstream prt_2("romb_extrap.dat");
  prt_1 << "Integration results using the Romberg method:\n";
  prt_1 << "\t for the integral int[e^x,x=0..1] = "
        << exact << "\n";
  prt_2 << "The Richardson extrapolates (up to 5) follow: \n";
  system("clear");
  while(doit)
  {
    cout << "Input the maximum number of extrapolations: \n";
    cout << "(a value of 1 to 20 is allowed)\n";
    cout << "\t nr = ? ";
    cin >> dnr;
    nr=int(dnr);
    nr=abs(nr);
    if(nr < 1 || nr > 20)
    {
      cout << "An incorrect value for nr has been entered---try again.\n";
      exit(0);
    }
    cout << "\n" << "We are using " << nr <<" extrapolations \n";
    h=(b - a);                       // Initial step size
    np=1;                            // Initial # of panels is one
    r[0][0]=0.5*h*(f(a) + f(b));     // The one panel result
    for (i=1; i<nr; i++)
    {
      h*=0.5;                        // Halve the step size
      np*=2;                         // Double the # of panels
      sum=0.0;                       // Initialize the integral
      for(k=1; k<np; k+=2)
      {
        x=a + double(k)*h;
        sum+=f(x);
      }
      r[i][0]=0.5*r[i-1][0] + h*sum; // Compute the first column r[i][0]
      m=1;
      for(j=1; j <= i; j++)          // Compute columns r[i][j]
      {
        m*=4;
        r[i][j]=r[i][j-1] + (r[i][j-1] - r[i-1][j-1])/double(m - 1);
      }
    }
    error=r[nr-1][nr-1] - exact;
    relerr=(error/exact)*100.0;
    cout << "For " << nr << " extrapolations," 
         << " Int = " << r[nr-1][nr-1] << " error = " << error
         << " %rel. error = " << relerr << "\n";
    cout << "The extended precision integral is \n";
    cout << "\t" << setprecision(16) << r[nr-1][nr-1] << "\n";
    prt_1 << "nr = " << setw(5) << nr
          << " Int = " << setw(10) << r[nr-1][nr-1] 
          << " error = " << setw(12) << error
          << " %rel. error = " << setw(12) << relerr << "\n";
    if(nr > 5)
     {
       nr=5;
     }
    for(i=0; i<nr; i++)
    {
      for(j=0; j <= i; j++)
      {
        prt_2 << setw(12) << r[i][j];
      }
      prt_2 << "\n";
    }
    prt_2 << "\n \n";
    cout << "\n \n" << "Change the number of extrapolations and try again? (y/n) ";
    cin >> ask;
    if(ask[0] == 'n' || ask[0] == 'N')
    {
    doit=0;
    }
  }
  return 0;
}   
//////////////////////////////////////////////////////////////////////////
// This function calculates the integrand
// for f(x)=e^x
//                            thomas l. gibson  April 25, 2002
//////////////////////////////////////////////////////////////////////////
double f(double t)
{
  double y;
  y=exp(t);
  return y;
}
