//////////////////////////////////////////////////////////////////////////
// This program codes an example of using the trapezoidal
// method to estimate the integral 
//
//          I = int[f(x),x=a..b],
//
// where f(x) is a user-defined function.  For this example
// we will assume f(x) is e^x and a=0 b=1.
//
//                thomas l. gibson  April 20, 2002
//
//     updated by thomas l. gibson  April 20, 2004
//////////////////////////////////////////////////////////////////////////
#include <cstdlib>               // Needed for system calls
#include <iostream>              // Needed for cout and cin
#include <fstream>               // Needed for file output
#include <iomanip>               // Needed for setw function in file out
#include <math.h>                // Needed for math functions
#include <cassert>               // Needed for assert
using namespace std;

double f(double t);              // Function prototype
int main()
{
  double a,b,h,error,relerr,sum,x,exact,dn,endpts;
  int n,i,doit;
  char ask[2];
  exact=exp(1.0) - 1.0;          // Exact value of integral
  doit=1;
  a=0.0;                         // Lower limit of integration
  b=1.0;                         // Upper limit of integration
  ofstream prt_1("trap_results.dat");
  prt_1 << "Integration results using the trapezoidal rule:\n";
  prt_1 << "\t for the integral int[e^x,x=0..1] = "
        << exact << "\n";
  system("clear");
  while(doit)
  {
    cout << "Input the number of panels for the Trapezoidal rule: \n";
    cout << "(a value of 1 to 512 is suggested)\n";
    cout << "\t n = ? ";
    cin >> dn;
    n=int(dn);
    n=abs(n);
    if(n == 0)
    {
      cout << "Using 0 panels is really boring!\n";
      exit(0);
    }
    cout << "\n" << "We are using " << n <<" panels \n";
    h=(b - a)/double(n);
    sum=0.0;
    endpts=0.5*h*(f(a) + f(b));   // Contribution from endpoints
    for (i=1; i<n; i++)
    {
      x=a + double(i)*h;
      sum+=f(x);
    }
    sum*=h;
    sum+=endpts;
    error=sum - exact;
    relerr=(error/exact)*100.0;
    cout << "For " << n << " panels, h = " << h 
         << " Int = " << sum << " error = " << error
         << " %rel. error = " << relerr << "\n";
    prt_1 << "n = " << setw(5) << n << " h = " << setw(10) << h
          << " Int = " << setw(10) << sum 
          << " error = " << setw(12) << error
          << " %rel. error = " << setw(12) << relerr << "\n";
    cout << "\n \n" << "Change the number of panels and try again? (y/n) ";
    cin >> ask;
    if(ask[0] == 'n' || ask[0] == 'N')
    {
    doit=0;
    }
  }
  return 0;
}   
//////////////////////////////////////////////////////////////////////////
// This function calculates the integrand
// for f(x)=e^x
//                     thomas l. gibson  April 20, 2002
//////////////////////////////////////////////////////////////////////////
double f(double t)
{
  double y;
  y=exp(t);
  return y;
}
